function [ d_b ] = bottleneckPD_flag_simplify( PD1_withInf, PD2_withInf, threshold, flag)
% bottleneckPD_noInf calculates the bottleneck distance between two
% persistent diagram PD1 and PD2, remove all persistence intervals with
% infinite ending
% flag - whether Inf are kept. flag = 1, kept. flag = 0, remove;
%
%   INPUT: PD1, PD2, two persistent diagrams in matrix format
%        : max_filtration_value - thresholding applied to filtration
%   OUTPUT: d_b bottlenect distance between two PDs
%
% Author: Weiyu Huang, whuang@seas.upenn.edu
% version 1
% 2017 Feb 17
% 
% References: W. Huang and A. Ribeiro, Persistent Homology Lower Bounds on
% High Order Network Distances, IEEE Transaction on Signal Processing, vol.
% 65, no. 2, pp. 319-334, January 2017.
%
% W. Huang and A. Ribeiro, Metrics in the Space of High Order Networks,
% IEEE Transaction on Signal Processing, vol. 64, no. 3, pp. 615-629,
% February 2016.


dim_1_withInf = size(PD1_withInf, 1);
dim_2_withInf = size(PD2_withInf, 1);

if max( dim_1_withInf, dim_2_withInf) == 0
    d_b = 0; return
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%% STEP 1-0: DELETE INTERVALS WITH INF ENDING %%%%%%%%%%%%%

if flag == 0 % remove all Inf
    PD1 = [];
    PD2 = [];
    
    for i = 1:dim_1_withInf
        if isfinite(PD1_withInf(i, 2))
            PD1 = cat(1, PD1, PD1_withInf(i,:));
        end
    end
    
    for i = 1:dim_2_withInf
        if isfinite(PD2_withInf(i, 2))
            PD2 = cat(1, PD2, PD2_withInf(i,:));
        end
    end
    
    dim_1 = size(PD1, 1);
    dim_2 = size(PD2, 1);
    d_max = max(dim_1, dim_2);
    
    if d_max == 0
        d_b = 0; return
    end
else % keep all inf, change to 110;
    PD1 = PD1_withInf;
    PD2 = PD2_withInf;
    PD1(isinf(PD1)) = 1100;
    PD2(isinf(PD2)) = 1100;
    
    dim_1 = size(PD1, 1);
    dim_2 = size(PD2, 1);
    d_max = max(dim_1, dim_2);
    
    if d_max == 0
        d_b = 0; return
    end
    
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%% STEP 1-1: SIMPLIFY %%%%%%%%%%%%%%%%%%%%%%%%%%%

temp = [];
for i = 1:size(PD1, 1)
    if PD1(i, 2) - PD1(i, 1) > threshold
        temp = cat(1, temp, PD1(i,:));
    end
end
PD1 = temp;

temp = [];
for i = 1:size(PD2, 1)
    if PD2(i, 2) - PD2(i, 1) > threshold
        temp = cat(1, temp, PD2(i,:));
    end
end
PD2 = temp;

dim_1 = size(PD1, 1);
dim_2 = size(PD2, 1);
d_max = max(dim_1, dim_2);

if d_max == 0
    d_b = 0; return
end

% [dim_1, dim_2]

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% STEP 1: ADD DIAGONAL TO MAKE BOTH PDS EQUAL IN NUMBER %%%%%%

if dim_1 < dim_2
    for i = 1:dim_2 - dim_1
        PD1 = cat(1, [0, 0], PD1);
    end
else if dim_2 < dim_1
    for i = 1:dim_1 - dim_2
        PD2 = cat(1, [0, 0], PD2);
    end
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%% STEP 2: COMPUTE COST MATRIX  %%%%%%%%%%%%%%%%%%%
   
Cost = zeros(d_max); % cost matrix
feasibility_check_executed = [];

for i = 1:d_max
    for j = 1:d_max
        Cost(i,j) = bottleneck_twoElements(PD1(i,:), PD2(j,:));
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%% STEP 3: THRESHOLD BOTTLENECK %%%%%%%%%%%%%%%%%%%

c0 = min(min(Cost)); % min
c1 = max(max(Cost)); % max

if c0 == c1
    d_b = c0; % any permutation is optimal
else
    while any( any( and( c0 < Cost, Cost < c1))) % check if C_star is empty
        Cost_star = Cost(  and( c0 < Cost, Cost < c1));
        Cost_star = sort(Cost_star, 'ascend');
        c_star = Cost_star( ceil( length(Cost_star)/2)); % median 
        
        [c_star, c0, c1] = threshold_feasibility_check...
            (Cost, c_star, c0, c1); 
        
        feasibility_check_executed =...
            cat(1, feasibility_check_executed, c_star); % store executed
    end
    
    if ~any(feasibility_check_executed(:) == c0)
        [~, ~, c1] = threshold_feasibility_check...
            (Cost, c0, c0, c1);
    end
    
    d_b = c1;

end

end

