% This codes use persistent homology to classify random networks generated
% with different models. We consider three models: Erdos-Ranyi random
% graph, Radial basis function, and correlation model.
%
% Author: Weiyu Huang, whuang@seas.upenn.edu
% version 1
% 2017 Feb 17
% 
% References: W. Huang and A. Ribeiro, Persistent Homology Lower Bounds on
% High Order Network Distances, IEEE Transaction on Signal Processing, vol.
% 65, no. 2, pp. 319-334, January 2017.
%
% W. Huang and A. Ribeiro, Metrics in the Space of High Order Networks,
% IEEE Transaction on Signal Processing, vol. 64, no. 3, pp. 615-629,
% February 2016.

%% STEP 1: create random networks
% codes can be modified to generate networks of different sizes

n = 30;
TRIAL = 50; % number of networks created

rng(13);
networks = cell(TRIAL * 3, 1);
for i = 1:TRIAL
    networks{i} = ER_weighted(n, 0.2, 0.2);
end
for i = TRIAL+1:2*TRIAL
    networks{i} = RBF(n, 0.5, 0.2);
end 
for i = 2*TRIAL+1:3*TRIAL
    networks{i} = correlation_model(n, 30, 0.2);
end

%% STEP2: compute their persistent homology feature

H0 = cell(TRIAL*3, 1);
H1 = cell(TRIAL*3, 1);

max_d1 = 0;
for i = 1:TRIAL
    max_d1 = max(max_d1, max(max(networks{i})));
end

for i = 1:TRIAL*3
    network = networks{i};
    
    % define the zeroth order and first order relationships for networks
    d1 = 1000 * zeros(n, 1);
    d2 = 1000 * (1 - network / max_d1) + 1; % features are measures of dissimilarity
    
    clear stream
    stream = api.Plex4.createExplicitSimplexStream();
    
    for j = 1:length(d1)
        stream.addVertex(j, d1(j));
    end
    
    for j = 1:length(d1)
        for k = 1:length(d1)
            stream.addElement([j, k], d2(j, k));
        end
    end
    stream.finalizeStream();
    
    % compute
    persistence = api.Plex4.getModularSimplicialAlgorithm(3,2);
    intervals = persistence.computeIntervals(stream);
    m_int = interval2Double(intervals);

    % store
    H0{i} = m_int.H0;
    H1{i} = m_int.H1;
end

%% STEP 3: compute pairwise distance between networks using homology

% distance between zeroth dimension homological features
distance_R0 = distance_matrix_bottleneck( H0, 0.1, 1 );

% distance via first dimension homological features
distance_R1 = distance_matrix_bottleneck( H1, 150, 1 );

%% STEP 4: Plot the MDS

D = distance_R1; % choose the one to deal with
D = D + 1e-13 * rand(length(D)); D = (D + D') / 2; D = D - diag(diag(D));

Y = mdscale(D, 2);

sig = ['or'; 'or'; 'or'; 'or'; 'or'; 'or'; 'or'; 'or'; 'or'; 'or'; ...
       'or'; 'or'; 'or'; 'or'; 'or'; 'or'; 'or'; 'or'; 'or'; 'or'; ...
       'or'; 'or'; 'or'; 'or'; 'or'; 'or'; 'or'; 'or'; 'or'; 'or'; ...
       'or'; 'or'; 'or'; 'or'; 'or'; 'or'; 'or'; 'or'; 'or'; 'or'; ...
       'or'; 'or'; 'or'; 'or'; 'or'; 'or'; 'or'; 'or'; 'or'; 'or'; ...
       'db'; 'db'; 'db'; 'db'; 'db'; 'db'; 'db'; 'db'; 'db'; 'db'; ...
       'db'; 'db'; 'db'; 'db'; 'db'; 'db'; 'db'; 'db'; 'db'; 'db'; ...
       'db'; 'db'; 'db'; 'db'; 'db'; 'db'; 'db'; 'db'; 'db'; 'db'; ...
       'db'; 'db'; 'db'; 'db'; 'db'; 'db'; 'db'; 'db'; 'db'; 'db'; ...
       'db'; 'db'; 'db'; 'db'; 'db'; 'db'; 'db'; 'db'; 'db'; 'db'; ...
       'sk'; 'sk'; 'sk'; 'sk'; 'sk'; 'sk'; 'sk'; 'sk'; 'sk'; 'sk'; ...
       'sk'; 'sk'; 'sk'; 'sk'; 'sk'; 'sk'; 'sk'; 'sk'; 'sk'; 'sk'; ...
       'sk'; 'sk'; 'sk'; 'sk'; 'sk'; 'sk'; 'sk'; 'sk'; 'sk'; 'sk'; ...
       'sk'; 'sk'; 'sk'; 'sk'; 'sk'; 'sk'; 'sk'; 'sk'; 'sk'; 'sk'; ...
       'sk'; 'sk'; 'sk'; 'sk'; 'sk'; 'sk'; 'sk'; 'sk'; 'sk'; 'sk'];
sig_switch = 1:length(Y);

handle = figure;
set(gca,'FontSize',14)
hold on; grid on;

for i = 1:length(Y)
    scatter( Y(i, 1), Y(i, 2), 90, sig(sig_switch(i),:), 'filled', 'LineWidth', 1.9);
end
    
set(gcf, 'position', [1000, 800, 560, 392])
